/***************************************************************************
 *
 * Copyright (c) 2002 Promise Technology, Inc.  All rights reserved.
 * No part of this document may be reproduced or transmitted in any form or
 * by any means, electronic or mechanical, for any purpose, without the
 * express written permission of Promise Technology, Inc.
 *
 *
 * cam_con.h - defines standard typedefs and other useful constants
 *
 ***************************************************************************/
 
#ifndef CAM_CON_H
#define CAM_CON_H
 
#ifndef bit
  	#define bit(x)  (1 << x)
#endif


#if defined (_WIN64)
	#ifndef _64BIT_
		#define _64BIT_
	#endif
#endif


#ifdef	_64BIT_
	#define	U64NULL				0xFFFFFFFFFFFFFFFF
#endif


#define U32NULL				0xFFFFFFFF
#define U16NULL				0xFFFF
#define U8NULL				0xFF
#define ZERO_NULL			0x0


#ifdef	_64BIT_
	#define	PTR_NULL	U64NULL
#else
	#define	PTR_NULL	U32NULL
#endif


#ifndef	TRUE
	#define TRUE	1
	#define FALSE	0
#endif

#define camSUCCESS					TRUE			
#define	camACCEPTED					TRUE
#define camFREE						TRUE
#define	camFAIL						FALSE
#define	camDEVICE_NOT_FOUND			FALSE
#define	camBUSY						FALSE
#define	camWARNING					0x02
#define	camCHNL_NOT_FOUND			0xFF
#define	camEnable					0x01
#define	camDisable					0x00


/* definitions for CAMFlag of CAM_INFO */	
#define IO_ACCESS		0x01			/* CAM will do IO access */
#define MEM_ACCESS		0x02			/* CAM will do memory-mapped IO access */
#define XOR_SUPPORTED	0x04			/* CAM support functions of XOR operation. */
#define HDMA_SUPPORTED	0x08			/* CAM support functions of HDMA operation. */
#define SERIAL_BUS		0x10			/* CAM support functions about serial bus. */


/* definitions for DeviceID CAM_INFO */
#define	CAM_PDC20620	0x6620		/* the CAM is for PDC20620/617 series ASIC */
#define CAM_PDC20621	0x6621		/* the CAM is for PDC20621 */
#define CAM_PDC20518	0x3518		/* support PDC20575/518 series ASIC (SATA150 II) */


/*  CAM base information */
#ifndef	OEM_PROMISE
	#define OEM_PROMISE		0
#endif

#ifndef	CAM_VerMajor
	#define CAM_VerMajor			0
	#define CAM_VerMinor			2
	#define CAM_VerOEM			OEM_PROMISE
	#define CAM_VerBuild			24
#endif

#ifdef	_MMIO_
	#define ACCESS_TYPE		(MEM_ACCESS + SERIAL_BUS)
#else
	#define	ACCESS_TYPE		(IO_ACCESS + SERIAL_BUS)
#endif
#define CAM_DEVICEID	CAM_PDC20518

#ifndef MAX_ADAPTER_NUM
	#define MAX_ADAPTER_NUM 			2			/* start from 1 adapter */
#endif

#define	MAX_ATA_CHNL				4 			/* The number of ATA channels per controller */
#ifdef Port_Multiplier
	#define MAX_ID_PER_CHNL				4			/* The number of ATA devices per channel */
#else
	#define MAX_ID_PER_CHNL				2			/* The number of ATA devices per channel */
#endif
#define MAX_XOR_MODULE				0			/* The number of XOR modules per controller */
#define MAX_HDMA_MODULE				0			/* The number of HDMA modules per controller */
#define	MAX_SEQ_ID				  	16			/* The number of SEQ ID per controller */
#define MAX_ATA_MODULE				(MAX_ADAPTER_NUM * MAX_ATA_CHNL)
#define MAX_ID_PER_ADAPTER   		(MAX_ATA_CHNL * MAX_ID_PER_CHNL) 
#ifndef NETWARE
  #define	START_SEQ_ID				1		/* SEQ ID 0 reserved to direct commands */
#else
  #define	START_SEQ_ID				8		/* NETWARE and BIOS should SEQs in differnt bytes */	
#endif

#define	INITI2CTIME			32

#define SREIAL_BUS_BUFFER_SIZE				0x10
#define SREIAL_BUS_POLLING_BUFFER_SIZE		0x10

/*  for memory and packet management */
#define MAXREGCNT		32		/* Must check that is enough for ATA and ATAPI both
								 * Then ATA packet size is 32 * N bytes 
								 */
#define MIN_ATA_PAC		0x1		/* MIN Packets per channel */
#define MAX_ATA_PAC		0x20	/* MAX Packets per channel */
#define MAX_ATA_RECEIVE_PAC		0x80	/* MAX Packets receive per channel */

#define	ATA_512K_BUFFER_ADDR	0x00
#define	SREIAL_BUS_BUFFER_ADDR	(ATA_512K_BUFFER_ADDR+512)
#define ATA_BASE_ADDR		(SREIAL_BUS_BUFFER_ADDR+((SREIAL_BUS_BUFFER_SIZE+SREIAL_BUS_POLLING_BUFFER_SIZE)*2))
#ifdef	_64BIT_
	#define MIN_CAM_MEMSIZE		(ATA_BASE_ADDR+(MAX_ATA_CHNL*MIN_ATA_PAC*(ATA_PAC_SIZE+ATA_SG_SIZE))+8)
	#define MAX_CAM_MEMSIZE		(ATA_BASE_ADDR+(MAX_ATA_CHNL*MAX_ATA_PAC*(ATA_PAC_SIZE+ATA_SG_SIZE))+8)
#else
	#define MIN_CAM_MEMSIZE		(ATA_BASE_ADDR+(MAX_ATA_CHNL*MIN_ATA_PAC*(ATA_PAC_SIZE+ATA_SG_SIZE))+4)
	#define MAX_CAM_MEMSIZE		(ATA_BASE_ADDR+(MAX_ATA_CHNL*MAX_ATA_PAC*(ATA_PAC_SIZE+ATA_SG_SIZE))+4)
#endif

#define CAM_PACKET_SIZE		(MAX_ATA_CHNL*(ATA_PAC_SIZE+ATA_SG_SIZE))

/* Registers */
#define	HOSTMEMBASE		0x000
#define ATAMEMBASE		0x200

#ifdef _MMIO_
#define ATA_PACSTART_REGISTER_OFFSET	0x40
#define ATA_STATUS_REGISTER_OFFSET		0x48
#else
#define ATA_PACSTART_REGISTER_OFFSET	0x00
#define ATA_STATUS_REGISTER_OFFSET		0x04
#endif

#define PACSTART_REGISTER_OFFSET	0x00
#define STATUS_REGISTER_OFFSET		0x04
#define INDEX_REGISTER_OFFSET		0x08
#define DATA_REGISTER_OFFSET		0x0C

/* Host Offset of ASIC */
#define	SEQINT_REGISTER_OFFSET			0x40

#define oFFSETHOSTFlashCTL				0x44
#define oFFSETHOSTPCICTL				0x48
#define oFFSETHOSTPLLCNT				0x4C
#define oFFSETHOSTEXTRA0				0x50
#define oFFSETHOSTEXTRA1				0x54
#define oFFSETHOSTEXTRA2				0x58
#define oFFSETMSICTL					0x5C
#define oFFSETHOSTSATAPLUG				0x60
	/* serial Bus */
#define oFFSETSBusControlStatus			0x64
#define oFFSETSBusMasterBaseAddr		0x68
#define oFFSETSBusCurrentMasterCMD		0x6C
#define oFFSETI2CSlaveReceiveByteCount	0x70
#define oFFSETSBusReceiveDataBaseAddr	0x74
#define oFFSETSBusUpperAddr				0x78

#define oFFSETTBGmode					0x7C
#define oFFSETSlewrate					0x80

/* IDE Index of ASIC */
#define iNDEXIDEData                0x00
#define iNDEXIDEFeature				0x04
#define iNDEXIDEError				0x04
#define iNDEXIDESectorCount		    0x08
#define iNDEXIDESectorNumber        0x0c
#define iNDEXIDECylinderLow         0x10
#define iNDEXIDECylinderHigh        0x14
#define iNDEXIDEDeviceHead          0x18
#define iNDEXIDECommand             0x1c
#define iNDEXIDEStatus				0x1c

#define iNDEXSwapBox				0x20
#define iNDEXIDEAlternateStatus     0x38
#define iNDEXIDEDeviceControl		0x38
#define iNDEXIDEPacketPointer		0x40
#define iNDEXIDEDestriptorPointer	0x44
#define iNDEXIDEDrvTiming0			0x4C
#define iNDEXIDEDrvTiming1			0x50
#define iNDEXIDECICR				0x60

#ifdef	_MMIO_
	#define iNDEXSATAStatus						0x200	/*0x400*/
	#define iNDEXIDESATAError					0x204   /*0x404*/
	#define iNDEXIDESATASControl				0x208   /*0x408*/
	#define iNDEXIDESATAControl					0x20C	/*0x40C*/
	#define iNDEXIDEPHYmode						0x214	/*0x414*/
	#define iNDEXIDETBGmode						0x21C	/*0x41C*/
	#define iNDEXIDEPHYQuality					0x22C	/*0x42C*/
	#define iNDEXIDEPHYMode2Reg				0x230
	#define iNDEXIDELatestFISD2H				0x24C
	#define iNDEXIDEMiscControlReg				0x250	/*0x450*/
	#define iNDEXIDEDWordDataPortforPIOAccess	0x268   /*0x468*/
	#define iNDEXIDEErrorsReportedfromLinkLayer	0x26C   /*0x46C*/
	#define iNDEXIDEMiscInternalControlState	0x27C   /*0x47C*/	
#else
	#define iNDEXSATAStatus						0x80
	#define iNDEXIDESATAError					0x84
	#define iNDEXIDESATASControl				0x88
	#define iNDEXIDESATAControl					0x8C
	#define iNDEXIDEPHYmode						0x94
	#define iNDEXIDETBGmode						0x9C
	#define iNDEXIDEPHYQuality					0xAC
	#define iNDEXIDEPHYMode2Reg				0xB0
	#define iNDEXIDELatestFISD2H				0xCC
	#define iNDEXIDEMiscControlReg				0xD0
	#define iNDEXIDEDWordDataPortforPIOAccess	0xE8
	#define iNDEXIDEErrorsReportedfromLinkLayer	0xEC
	#define iNDEXIDEMiscInternalControlState		0xFC	
#endif

/* iNDEXIDECICR Register definitions */
#define IDECICR_DMA_RW_Direction			bit(6)
#define IDECICR_DMA_Start					bit(7)

/* Port Multiplier */
#define PM_control_Port	0x0F


/* IDE Index_exp of ASIC */
#define iNDEXeXP_FPDMA_Vector_Table_Base	0x00
#define iNDEXeXP_FPDMA_Packet_High_Addr		0x40
#define iNDEXeXP_FPDMA_PRD_High_Addr		0x44
#define iNDEXeXP_FPDMA_Ext_Ctrl				0x50
#define iNDEXeXP_FPDMA_Encry_SeedKey			0x54
#define iNDEXeXP_FPDMA_Ctrl_Status			0x58
#define iNDEXeXP_SActive					0x5C
#define iNDEXeXP_PIO_Transfer_count			0x60
#define iNDEXeXP_PM_Port					0x68
#define iNDEXeXP_Internel_debug_1			0x78
#define iNDEXeXP_Internel_debug_2			0x7C

/* iNDEXeXP_FPDMA_Ctrl_Status Register definitions */
#define FPDMA_Ctrl_Status_FPDMA_auto			bit(1)
#define FPDMA_Ctrl_Status_Packet_PRD			bit(2)
#define FPDMA_Ctrl_Status_Reset_FPDMA			bit(3)
#define FPDMA_Ctrl_Status_FPDMA_direction		bit(6)
#define FPDMA_Ctrl_Status_FPDMA_start_stop		bit(7)
#define FPDMA_Ctrl_Status_DMAsetup_INT_mask		bit(8)
#define FPDMA_Ctrl_Status_SetDb_INT_mask		bit(9)
#define FPDMA_Ctrl_Status_DMAsetup_INT_flag		bit(10)
#define FPDMA_Ctrl_Status_SetDb_INT_flag		bit(11)
#define FPDMA_Ctrl_Status_DMA_RW_direction		bit(21)
#define FPDMA_Ctrl_Status_Auto_active			bit(22)

#define IDE_SectorCount_RELEASE			0x04

/* IDE status definitions */
#define IDE_STATUS_ERROR			0x01
#define IDE_STATUS_INDEX			0x02
#define IDE_STATUS_CORRECTED_ERROR	0x04
#define IDE_STATUS_DRQ				0x08
#define IDE_STATUS_DSC				0x10
#define IDE_STATUS_SERVICE			0x10
#define IDE_STATUS_DF				0x20	/* Device Fault */
#define IDE_STATUS_DRDY				0x40
#define IDE_STATUS_IDLE				0x50
#define IDE_STATUS_BUSY				0x80

/* SATA Error Register status definitions */
#define DATA_ERROR_FIXED		0x00000001		/* bit 0 */
#define COMM_ERR_FIXED		0x00000002		/* bit 1 */
#define DATA_ERR				0x00000100		/* bit 8 */
#define PRESISTENT_ERR		0x00000200		/* bit 9 */
#define PROTOCOL_ERR			0x00000400		/* bit 10 */
#define COMWAKE				0x00040000		/* bit 18 */
#define Decode_10B_TO_8B_ERR	0x00080000		/* bit 19 */
#define DISPARITY_ERR			0x00100000		/* bit 20 */
#define CRC_ERR				0x00200000		/* bit 21 */
#define R_ERR					0x00400000		/* bit 22 */
#define LINK_SEQ_ERR			0x00800000		/* bit 23 */
#define FIS_TYP				0x02000000		/* bit 25 */
/* other bits reserved */

#define ATRLREG_REG_SIZE_8                  (0x00)
#define ATRLREG_REG_SIZE_7                  (0x07 << 5)
#define ATRLREG_REG_SIZE_6                  (0x06 << 5)
#define ATRLREG_REG_SIZE_5                  (0x05 << 5)
#define ATRLREG_REG_SIZE_4                  (0x04 << 5)
#define ATRLREG_REG_SIZE_3                  (0x03 << 5)
#define ATRLREG_REG_SIZE_2                  (0x02 << 5)
#define ATRLREG_REG_SIZE_1                  (0x01 << 5)

#define ATRLREG_CONDITION_NO                0x00
#define ATRLREG_CONDITION_CHECK_NOT_BUSY    0x10
#define ATRLREG_CONDITION_CHECK_DRQ         0x18
#define ATRLREG_CONDITION_X_DEFINE          0x08

#define ATRLREG_X_3F6                       0x0E
#define ATRLREG_X_END_1F7                   0x0F
#define ATRLREG_X_END_1F0                   0x08
#define ATRLREG_X_END_1FX                   0x08

#define ATRLREG_DATA                        0x00
#define ATRLREG_FEATURE                     0x01
#define ATRLREG_SECTOR_COUNT                0x02
#define ATRLREG_LBA0_7                      0x03
#define ATRLREG_LBA8_15                     0x04
#define ATRLREG_LBA16_23                    0x05
#define ATRLREG_LBA24_27                    0x06
#define ATRLREG_COMMAND                     0x07

#define ATRLREG_SECTORNUM                   0x03
#define ATRLREG_LBALOW                      0x04
#define ATRLREG_LBAHIGH                     0x05
#define ATRLREG_DEVICESELECT                0x06

#define SEQ_NO_SEQID            			0xFF
#define SEQ_NO_DELAY            			0xFF

/*
 * IDE command definitions
 */
#define IDE_COMMAND_ATAPI_RESET                     	0x08	
#define IDE_COMMAND_CFA_ERASE_SECTORS					0xC0
#define IDE_COMMAND_CFA_REQUEST_EXTENDED_ERROR			0x03
#define IDE_COMMAND_CFA_TRANSLATE_SECTOR				0x87
#define IDE_COMMAND_CFA_WRITE_MULTIPLE_WITHOUT_ERASE	0xCD
#define IDE_COMMAND_CFA_WRITE_SECTORS_WITHOUT_ERASE		0x38
#define IDE_COMMAND_CHECK_MEDIA_CARD_TYPE				0xD1
#define IDE_COMMAND_CHECK_POWER_MODE					0xE5
#define IDE_COMMAND_CONFIGURE_STREAM					0x51
#define IDE_COMMAND_DEVICE_CONFIGURATION				0xB1
/*
#define IDE_COMMAND_DEVICE_CONFIGURATION_FREEZE_LOCK	0xB1
#define IDE_COMMAND_DEVICE_CONFIGURATION_IDENTIFY		0xB1
#define IDE_COMMAND_DEVICE_CONFIGURATION_RESTORE		0xB1
#define IDE_COMMAND_DEVICE_CONFIGURATION_SET			0xB1
*/
#define IDE_COMMAND_DOWNLOAD_MICROCODE					0x92
#define IDE_COMMAND_EXECUTE_DEVICE_DIAGNOSTIC			0x90
#define IDE_COMMAND_FLUSH_CACHE							0xE7
#define IDE_COMMAND_FLUSH_CACHE_EXT						0xEA
#define IDE_COMMAND_GET_MEDIA_STATUS					0xDA
#define IDE_COMMAND_IDENTIFY_DEVICE						0xEC
#define IDE_COMMAND_IDENTIFY_PACKET_DEVICE				0xA1
#define IDE_COMMAND_IDLE								0xE3
#define IDE_COMMAND_IDLE_IMMEDIATE						0xE1
#define IDE_COMMAND_MEDIA_EJECT							0xED
#define IDE_COMMAND_MEDIA_LOCK							0xDE
#define IDE_COMMAND_MEDIA_UNLOCK						0xDF
#define IDE_COMMAND_NOP									0x00
#define IDE_COMMAND_PACKET								0xA0
#define IDE_COMMAND_READ_BUFFER							0xE4
#define IDE_COMMAND_READ_DMA							0xC8
#define IDE_COMMAND_READ_DMA_EXT						0x25
#define IDE_COMMAND_READ_DMA_QUEUED						0xC7
#define IDE_COMMAND_READ_DMA_QUEUED_EXT					0x26
#define IDE_COMMAND_READ_FPDMA_QUEUED					0x60	/* NCQ */
#define IDE_COMMAND_READ_LOG_EXT						0x2F
#define IDE_COMMAND_READ_LONG						0x22
#define IDE_COMMAND_READ_MULTIPLE						0xC4
#define IDE_COMMAND_READ_MULTIPLE_EXT					0x29
#define IDE_COMMAND_READ_NATIVE_MAX_ADDRESS				0xF8
#define IDE_COMMAND_READ_NATIVE_MAX_ADDRESS_EXT			0x27
#define IDE_COMMAND_READ_PORT_MULTIPLIER				0xE4	/* Port Multiplier */
#define IDE_COMMAND_READ_SECTORS						0x20
#define IDE_COMMAND_READ_SECTORS_EXT					0x24
#define IDE_COMMAND_READ_STREAM_DMA						0x2A
#define IDE_COMMAND_READ_STREAM_PIO						0x2B
#define IDE_COMMAND_READ_VERIFY_SECTORS					0x40
#define IDE_COMMAND_READ_VERIFY_SECTORS_EXT				0x42
#define IDE_COMMAND_RECALIBRATE                     	0x10	
#define IDE_COMMAND_SECURITY_DISABLE_PASSWORD			0xF6
#define IDE_COMMAND_SECURITY_ERASE_PREPARE				0xF3
#define IDE_COMMAND_SECURITY_ERASE_UNIT					0xF4
#define IDE_COMMAND_SECURITY_FREEZE_LOCK				0xF5
#define IDE_COMMAND_SECURITY_SET_PASSWORD				0xF1
#define IDE_COMMAND_SECURITY_UNLOCK						0xF2
#define IDE_COMMAND_SEEK                            	0x70
#define IDE_COMMAND_SERVICE								0xA2
#define IDE_COMMAND_SET_DRIVE_PARAMETERS    			0x91
#define IDE_COMMAND_SET_FEATURES						0xEF
#define IDE_COMMAND_SET_MAX_ADDRESS						0xF9
#define IDE_COMMAND_SET_MAX_ADDRESS_EXT					0x37
#define IDE_COMMAND_SET_MULTIPLE_MODE					0xC6
#define IDE_COMMAND_SLEEP								0xE6
#define IDE_COMMAND_SMART_CMD		           			0xB0
/*
#define IDE_COMMAND_SMART_DISABLE_OPERATIONS			0xB0
#define IDE_COMMAND_SMART_ENABLE_DISABLE_AUTOSAVE		0xB0
#define IDE_COMMAND_SMART_ENABLE_OPERATIONS				0xB0
#define IDE_COMMAND_SMART_EXECUTE_OFF_LINE				0xB0
#define IDE_COMMAND_SMART_READ_DATA						0xB0
#define IDE_COMMAND_SMART_READ_LOG_SECTOR				0xB0
#define IDE_COMMAND_SMART_RETURN_STATUS					0xB0
#define IDE_COMMAND_SMART_WRITE_LOG_SECTOR				0xB0
*/

#define IDE_COMMAND_STANDBY								0xE2
#define IDE_COMMAND_STANDBY_IMMEDIATE					0xE0
#define IDE_COMMAND_WRITE_BUFFER						0xE8
#define IDE_COMMAND_WRITE_DMA							0xCA
#define IDE_COMMAND_WRITE_DMA_EXT						0x35
#define IDE_COMMAND_WRITE_DMA_FUA_EXT					0x3D
#define IDE_COMMAND_WRITE_DMA_QUEUED					0xCC
#define IDE_COMMAND_WRITE_DMA_QUEUED_EXT				0x36
#define IDE_COMMAND_WRITE_DMA_QUEUED_FUA_EXT			0x3E
#define IDE_COMMAND_WRITE_FPDMA_QUEUED					0x61	/* NCQ */
#define IDE_COMMAND_WRITE_LOG_EXT						0x3F
#define IDE_COMMAND_WRITE_LONG						0x32
#define IDE_COMMAND_WRITE_MULTIPLE						0xC5
#define IDE_COMMAND_WRITE_MULTIPLE_EXT					0x39
#define IDE_COMMAND_WRITE_MULTIPLE_FUA_EXT				0xCE
#define IDE_COMMAND_WRITE_PORT_MULTIPLIER				0xE8	/* Port Multiplier */
#define IDE_COMMAND_WRITE_SECTORS						0x30
#define IDE_COMMAND_WRITE_SECTORS_EXT					0x34
#define IDE_COMMAND_WRITE_STREAM_DMA					0x3A
#define IDE_COMMAND_WRITE_STREAM_PIO					0x3B




/*
 * ATAPI command definitions
 */

#define REQUEST_SENSE_COMMAND                     	0x03

/* Int reason	*/
#define INT_WAITCMD			0xF1		/* Ready to accept command packet */
#define INT_DATAOUT			0x00		/* Data to Host */
#define INT_DATAIN			0x02		/* Data From Host */
#define INT_COMPELTE		0x03		/* Command Complete */


/* queue direction */
#define QUEUEHead 0x00
#define QUEUETail 0x01


/* Enclosure */
#define WINBONDW83783SID	0x5CA3
#define WINBONDW83782DID	0x5CA3
#define DEF_FANDIV	2
#define	DEF_CHKREPTIME		(18*10)	/* 10 seconds  */
#define	DEF_CHKSTATETIME	3	/* 30 seconds */
#define	NEW_DRIVE	TRUE
#define	NO_DRIVE	FALSE

#define	I2C_DELAY_COMMAND_COUNT	10

/* Timeout Value */
#define DEF_ATATIMEOUT		(18*5)	/* 5 seconds  */
#define DEF_ATAPITIMEOUT	(18*20)	/* 20 seconds  */

/* SATA drive initial count */
#define DEF_SATA_INITCOUNT	5

#define MAX_BAY_PER_BOX		4
#define MAX_I2C_PAC			(MAX_ATA_CHNL*0x10)

/* SATA NCQ MAX queue depth. */
#define MAX_NCQ_QUEUE_DEPTH		MAX_SEQ_ID;	/* 16 */

/* PHY reset */
#define PHY_Enable_RetryTime	4
#define PHY_Wait_Ready_Time	10

/* for ASIC bug, limit the last element of SG byteCount must <= 41 Dword */
#define SG_COUNT_ASIC_BUG	41*4
//#define SG_COUNT_ASIC_BUG	32
//#define SG_COUNT_ASIC_BUG	128

#endif
